@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with GNU SASL Library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -24,59 +24,65 @@
 # include "config.h"
 #endif
 
+/* Get specification. */
+#include "session.h"
+
 /* Get malloc, free. */
 #include <stdlib.h>
 
 /* Get memcpy, strdup, strlen. */
 #include <string.h>
 
-/* Get gsasl.h and other stuff. */
-#include "shared.h"
+/* Get gc_hmac_md5. */
+#include <gc.h>
 
-/* Get specification. */
-#include "session.h"
+#define MD5LEN 16
+#define SASL_INTEGRITY_PREFIX_LENGTH 4
+#define MAC_DATA_LEN 4
+#define MAC_HMAC_LEN 10
+#define MAC_MSG_TYPE "\x00\x01"
+#define MAC_MSG_TYPE_LEN 2
+#define MAC_SEQNUM_LEN 4
 
 int
-digest_md5_encode (Gsasl_session * sctx,
-		   const char *input, size_t input_len,
+digest_md5_encode (const char *input, size_t input_len,
 		   char **output, size_t * output_len,
-		   Gsasl_qop qop,
-		   uint32_t sendseqnum,
-		   char key[MD5LEN])
+		   digest_md5_qop qop,
+		   unsigned long sendseqnum, char key[DIGEST_MD5_LENGTH])
 {
   int res;
 
-  if (qop & GSASL_QOP_AUTH_CONF)
+  if (qop & DIGEST_MD5_QOP_AUTH_CONF)
     {
-      return GSASL_INTEGRITY_ERROR;
+      return -1;
     }
-  else if (qop & GSASL_QOP_AUTH_INT)
+  else if (qop & DIGEST_MD5_QOP_AUTH_INT)
     {
       char *seqnumin;
-      char *hash;
-      uint32_t tmp;
+      char hash[GC_MD5_LEN];
       size_t len;
 
       seqnumin = malloc (MAC_SEQNUM_LEN + input_len);
       if (seqnumin == NULL)
-	return GSASL_MALLOC_ERROR;
+	return -1;
 
-      tmp = htonl (sendseqnum);
-      memcpy (seqnumin, (char *) &tmp, MAC_SEQNUM_LEN);
+      seqnumin[0] = (sendseqnum >> 24) & 0xFF;
+      seqnumin[1] = (sendseqnum >> 16) & 0xFF;
+      seqnumin[2] = (sendseqnum >> 8) & 0xFF;
+      seqnumin[3] = sendseqnum & 0xFF;
       memcpy (seqnumin + MAC_SEQNUM_LEN, input, input_len);
 
-      res = gsasl_hmac_md5 (key, MD5LEN,
-			    seqnumin, MAC_SEQNUM_LEN + input_len,
-			    (char **) &hash);
+      res = gc_hmac_md5 (key, MD5LEN,
+			 seqnumin, MAC_SEQNUM_LEN + input_len, hash);
       free (seqnumin);
-      if (res != GSASL_OK || hash == NULL)
-	return GSASL_CRYPTO_ERROR;
+      if (res)
+	return -1;
 
       *output_len = MAC_DATA_LEN + input_len + MAC_HMAC_LEN +
 	MAC_MSG_TYPE_LEN + MAC_SEQNUM_LEN;
       *output = malloc (*output_len);
       if (!*output)
-	return GSASL_MALLOC_ERROR;
+	return -1;
 
       len = MAC_DATA_LEN;
       memcpy (*output + len, input, input_len);
@@ -85,71 +91,78 @@ digest_md5_encode (Gsasl_session * sctx,
       len += MAC_HMAC_LEN;
       memcpy (*output + len, MAC_MSG_TYPE, MAC_MSG_TYPE_LEN);
       len += MAC_MSG_TYPE_LEN;
-      tmp = htonl (sendseqnum);
-      memcpy (*output + len, &tmp, MAC_SEQNUM_LEN);
+      (*output + len)[0] = (sendseqnum >> 24) & 0xFF;
+      (*output + len)[1] = (sendseqnum >> 16) & 0xFF;
+      (*output + len)[2] = (sendseqnum >> 8) & 0xFF;
+      (*output + len)[3] = sendseqnum & 0xFF;
       len += MAC_SEQNUM_LEN;
-      tmp = htonl (len - MAC_DATA_LEN);
-      memcpy (*output, &tmp, MAC_DATA_LEN);
-
-      free (hash);
+      (*output)[0] = ((len - MAC_DATA_LEN) >> 24) & 0xFF;
+      (*output)[1] = ((len - MAC_DATA_LEN) >> 16) & 0xFF;
+      (*output)[2] = ((len - MAC_DATA_LEN) >> 8) & 0xFF;
+      (*output)[3] = (len - MAC_DATA_LEN) & 0xFF;
     }
   else
     {
       *output_len = input_len;
       *output = malloc (input_len);
       if (!*output)
-	return GSASL_MALLOC_ERROR;
+	return -1;
       memcpy (*output, input, input_len);
     }
 
-  return GSASL_OK;
+  return 0;
 }
 
+#define C2I(buf) ((buf[0] & 0xFF) |		\
+		  ((buf[1] & 0xFF) << 8) |	\
+		  ((buf[2] & 0xFF) << 16) |	\
+		  ((buf[3] & 0xFF) << 24))
+
 int
-digest_md5_decode (Gsasl_session * sctx,
-		   const char *input,
-		   size_t input_len,
+digest_md5_decode (const char *input, size_t input_len,
 		   char **output, size_t * output_len,
-		   Gsasl_qop qop,
-		   uint32_t readseqnum,
-		   char key[MD5LEN])
+		   digest_md5_qop qop,
+		   unsigned long readseqnum, char key[DIGEST_MD5_LENGTH])
 {
-  if (qop & GSASL_QOP_AUTH_CONF)
+  if (qop & DIGEST_MD5_QOP_AUTH_CONF)
     {
-      return GSASL_INTEGRITY_ERROR;
+      return -1;
     }
-  else if (qop & GSASL_QOP_AUTH_INT)
+  else if (qop & DIGEST_MD5_QOP_AUTH_INT)
     {
       char *seqnumin;
-      char *hash;
-      uint32_t len, tmp;
+      char hash[GC_MD5_LEN];
+      unsigned long len;
+      char tmpbuf[SASL_INTEGRITY_PREFIX_LENGTH];
       int res;
 
       if (input_len < SASL_INTEGRITY_PREFIX_LENGTH)
-	return GSASL_NEEDS_MORE;
+	return -2;
 
-      len = ntohl (*(uint32_t *) input);
+      len = C2I (input);
 
       if (input_len < SASL_INTEGRITY_PREFIX_LENGTH + len)
-	return GSASL_NEEDS_MORE;
+	return -2;
 
       len -= MAC_HMAC_LEN + MAC_MSG_TYPE_LEN + MAC_SEQNUM_LEN;
 
       seqnumin = malloc (SASL_INTEGRITY_PREFIX_LENGTH + len);
       if (seqnumin == NULL)
-	return GSASL_MALLOC_ERROR;
+	return -1;
 
-      tmp = htonl (readseqnum);
+      tmpbuf[0] = (readseqnum >> 24) & 0xFF;
+      tmpbuf[1] = (readseqnum >> 16) & 0xFF;
+      tmpbuf[2] = (readseqnum >> 8) & 0xFF;
+      tmpbuf[3] = readseqnum & 0xFF;
 
-      memcpy (seqnumin, (char *) &tmp, SASL_INTEGRITY_PREFIX_LENGTH);
+      memcpy (seqnumin, tmpbuf, SASL_INTEGRITY_PREFIX_LENGTH);
       memcpy (seqnumin + SASL_INTEGRITY_PREFIX_LENGTH,
 	      input + MAC_DATA_LEN, len);
 
-      res = gsasl_hmac_md5 (key, MD5LEN, seqnumin, MAC_SEQNUM_LEN + len,
-			    (char **) &hash);
+      res = gc_hmac_md5 (key, MD5LEN, seqnumin, MAC_SEQNUM_LEN + len, hash);
       free (seqnumin);
-      if (res != GSASL_OK || hash == NULL)
-	return GSASL_CRYPTO_ERROR;
+      if (res)
+	return -1;
 
       if (memcmp
 	  (hash,
@@ -158,29 +171,26 @@ digest_md5_decode (Gsasl_session * sctx,
 	  && memcmp (MAC_MSG_TYPE,
 		     input + input_len - MAC_SEQNUM_LEN - MAC_MSG_TYPE_LEN,
 		     MAC_MSG_TYPE_LEN) == 0
-	  && memcmp (&tmp, input + input_len - MAC_SEQNUM_LEN,
+	  && memcmp (tmpbuf, input + input_len - MAC_SEQNUM_LEN,
 		     MAC_SEQNUM_LEN) == 0)
 	{
 	  *output_len = len;
 	  *output = malloc (*output_len);
 	  if (!*output)
-	    return GSASL_MALLOC_ERROR;
+	    return -1;
 	  memcpy (*output, input + MAC_DATA_LEN, len);
 	}
       else
-	return GSASL_INTEGRITY_ERROR;
-
-      free (hash);
+	return -1;
     }
   else
     {
       *output_len = input_len;
       *output = malloc (input_len);
       if (!*output)
-	return GSASL_MALLOC_ERROR;
+	return -1;
       memcpy (*output, input, input_len);
     }
 
-
-  return GSASL_OK;
+  return 0;
 }
