@@ -1,5 +1,5 @@
 /* init.c --- Entry point for libgsasl.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -13,9 +13,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * License along with GNU SASL Library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License License along with GNU SASL Library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -37,13 +38,19 @@
 #include "ntlm/x-ntlm.h"
 #include "kerberos_v5/kerberos_v5.h"
 
+/**
+ * GSASL_VALID_MECHANISM_CHARACTERS:
+ *
+ * A zero-terminated character array, or string, with all ASCII
+ * characters that may be used within a SASL mechanism name.
+ **/
 const char *GSASL_VALID_MECHANISM_CHARACTERS =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_";
 
 static int
 register_builtin_mechs (Gsasl * ctx)
 {
-  int rc;
+  int rc = GSASL_OK;
 
 #ifdef USE_ANONYMOUS
   rc = gsasl_register (ctx, &gsasl_anonymous_mechanism);
@@ -81,18 +88,18 @@ register_builtin_mechs (Gsasl * ctx)
     return rc;
 #endif /* USE_NTLM */
 
-#ifdef USE_CRAM_MD5
-  rc = gsasl_register (ctx, &gsasl_cram_md5_mechanism);
-  if (rc != GSASL_OK)
-    return rc;
-#endif /* USE_CRAM_MD5 */
-
 #ifdef USE_DIGEST_MD5
   rc = gsasl_register (ctx, &gsasl_digest_md5_mechanism);
   if (rc != GSASL_OK)
     return rc;
 #endif /* USE_DIGEST_MD5 */
 
+#ifdef USE_CRAM_MD5
+  rc = gsasl_register (ctx, &gsasl_cram_md5_mechanism);
+  if (rc != GSASL_OK)
+    return rc;
+#endif /* USE_CRAM_MD5 */
+
 #ifdef USE_GSSAPI
   rc = gsasl_register (ctx, &gsasl_gssapi_mechanism);
   if (rc != GSASL_OK)
@@ -116,7 +123,6 @@ register_builtin_mechs (Gsasl * ctx)
 int
 gsasl_init (Gsasl ** ctx)
 {
-  size_t i;
   int rc;
 
   if (gc_init () != GC_OK)
